$VAR1 = {
          'stuff' => [
                     {
                       'name' => 'Moose',
                       'other' => 'Value'
                     },
                     {
                       'name' => 'Thingy',
                       'other' => 'Value2'
                     }
                   ]
        };
