$VAR1 = {
          'stuff' => [
                     {
                       'name' => 'Moose',
                       'other' => {
                                   content => 'Value',
                                   attrib => 'first'
                                  }
                     },
                     {
                       'name' => 'Thingy',
                       'other' => {
                                   content => 'Value2',
                                   attrib => 'second'
                                  }
                     }
                   ]
        };
