$VAR1 = {
          'stuff' => {
                     'first' => {
                                'name' => 'Moose',
                                'attrib' => 'first'
                              },
                     'second' => {
                                 'name' => 'Thingy',
                                 'attrib' => 'second'
                               }
                   }
        };
