$VAR1 = {
          'stuff' => [
                     {
                       'name' => 'Moose',
                       'attrib' => 'first'
                     },
                     {
                       'name' => 'Thingy',
                       'attrib' => 'second'
                     }
                   ]
        };
