$VAR1 = {
          'spotgroup' => [
                         {
                           'date' => '2003-01',
                           'spotlight' => [
                                          {
                                            'alttag' => 'Artists Behind the Desk',
                                            'text' => 'The Artists Behind the Desk Committee proudly presents
        a Visual Arts Exhibition featuring works in a variety of media
        from over 30 artists who divide their time between creating
        art and working.',
                                            'url' => 'http://example.com/abd/',
                                            'img' => '../images/spotlight/drag.jpg',
                                            'title' => 'This Friday',
                                            'seealso' => [
                                                         {
                                                           'catid' => 'arts'
                                                         }
                                                       ]
                                          }
                                        ]
                         },
                         {
                           'date' => '2003-03',
                           'spotlight' => [
                                          {
                                            'text' => 'Check out the new Sports and Fitness Center!',
                                            'url' => 'http://example.com/projects/zesiger/',
                                            'img' => '../images/spotlight/center.jpg',
                                            'title' => 'Sports and Fitness Center',
                                            'seealso' => [
                                                         {
                                                           'text' => 'Athletics',
                                                           'url' => 'http://example.com/athletics/'
                                                         },
                                                         {
                                                           'text' => 'Campus Projects',
                                                           'url' => 'http://example.com/projects/'
                                                         }
                                                       ]
                                          }
                                        ]
                         }
                       ]
        };
